@@ -8,7 +8,7 @@ use std::{
     ptr, slice,
 };
 
-use glib::translate::*;
+use glib::{prelude::*, translate::*};
 
 use crate::{ffi, AllocationParams, Allocator, MemoryFlags};
 
@@ -17,15 +17,13 @@ mini_object_wrapper!(Memory, MemoryRef, ffi::GstMemory, || {
 });
 
 pub struct MemoryMap<'a, T> {
-    memory: &'a MemoryRef,
     map_info: ffi::GstMapInfo,
-    phantom: PhantomData<T>,
+    phantom: PhantomData<(&'a MemoryRef, T)>,
 }
 
 pub struct MappedMemory<T> {
-    memory: Memory,
     map_info: ffi::GstMapInfo,
-    phantom: PhantomData<T>,
+    phantom: PhantomData<(Memory, T)>,
 }
 
 impl fmt::Debug for Memory {
@@ -53,6 +51,25 @@ pub enum Readable {}
 pub enum Writable {}
 
 impl Memory {
+    #[inline]
+    pub fn default_alignment() -> usize {
+        #[cfg(not(windows))]
+        {
+            extern "C" {
+                static gst_memory_alignment: usize;
+            }
+            unsafe { gst_memory_alignment }
+        }
+        #[cfg(windows)]
+        {
+            // FIXME: Windows is special and accessing variables exported from
+            // shared libraries need different handling than from static libraries
+            // so just return the default MEMORY_ALIGNMENT_MALLOC here until someone
+            // figures out how to do this properly on Windows.
+            7
+        }
+    }
+
     #[inline]
     pub fn with_size(size: usize) -> Self {
         assert_initialized_main_thread!();
@@ -80,20 +97,20 @@ impl Memory {
     #[inline]
     pub fn into_mapped_memory_readable(self) -> Result<MappedMemory<Readable>, Self> {
         unsafe {
+            let s = mem::ManuallyDrop::new(self);
             let mut map_info = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_memory_map(
-                self.as_mut_ptr(),
+                s.as_mut_ptr(),
                 map_info.as_mut_ptr(),
                 ffi::GST_MAP_READ,
             ));
             if res {
                 Ok(MappedMemory {
-                    memory: self,
                     map_info: map_info.assume_init(),
                     phantom: PhantomData,
                 })
             } else {
-                Err(self)
+                Err(mem::ManuallyDrop::into_inner(s))
             }
         }
     }
@@ -101,20 +118,20 @@ impl Memory {
     #[inline]
     pub fn into_mapped_memory_writable(self) -> Result<MappedMemory<Writable>, Self> {
         unsafe {
+            let s = mem::ManuallyDrop::new(self);
             let mut map_info = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_memory_map(
-                self.as_mut_ptr(),
+                s.as_mut_ptr(),
                 map_info.as_mut_ptr(),
                 ffi::GST_MAP_READWRITE,
             ));
             if res {
                 Ok(MappedMemory {
-                    memory: self,
                     map_info: map_info.assume_init(),
                     phantom: PhantomData,
                 })
             } else {
-                Err(self)
+                Err(mem::ManuallyDrop::into_inner(s))
             }
         }
     }
@@ -128,7 +145,7 @@ impl MemoryRef {
             if self.0.allocator.is_null() {
                 None
             } else {
-                Some(&*(&self.0.allocator as *const *mut ffi::GstAllocator as *const Allocator))
+                Some(Allocator::from_glib_ptr_borrow(&self.0.allocator))
             }
         }
     }
@@ -271,14 +288,13 @@ impl MemoryRef {
     }
 
     #[inline]
-    pub fn map_readable(&self) -> Result<MemoryMap<Readable>, glib::BoolError> {
+    pub fn map_readable(&self) -> Result<MemoryMap<'_, Readable>, glib::BoolError> {
         unsafe {
             let mut map_info = mem::MaybeUninit::uninit();
             let res =
                 ffi::gst_memory_map(self.as_mut_ptr(), map_info.as_mut_ptr(), ffi::GST_MAP_READ);
             if res == glib::ffi::GTRUE {
                 Ok(MemoryMap {
-                    memory: self,
                     map_info: map_info.assume_init(),
                     phantom: PhantomData,
                 })
@@ -289,7 +305,7 @@ impl MemoryRef {
     }
 
     #[inline]
-    pub fn map_writable(&mut self) -> Result<MemoryMap<Writable>, glib::BoolError> {
+    pub fn map_writable(&mut self) -> Result<MemoryMap<'_, Writable>, glib::BoolError> {
         unsafe {
             let mut map_info = mem::MaybeUninit::uninit();
             let res = ffi::gst_memory_map(
@@ -299,7 +315,6 @@ impl MemoryRef {
             );
             if res == glib::ffi::GTRUE {
                 Ok(MemoryMap {
-                    memory: self,
                     map_info: map_info.assume_init(),
                     phantom: PhantomData,
                 })
@@ -334,7 +349,7 @@ impl MemoryRef {
     }
 
     #[doc(alias = "gst_util_dump_mem")]
-    pub fn dump(&self) -> Dump {
+    pub fn dump(&self) -> Dump<'_> {
         Dump {
             memory: self,
             start: Bound::Unbounded,
@@ -343,7 +358,7 @@ impl MemoryRef {
     }
 
     #[doc(alias = "gst_util_dump_mem")]
-    pub fn dump_range(&self, range: impl RangeBounds<usize>) -> Dump {
+    pub fn dump_range(&self, range: impl RangeBounds<usize>) -> Dump<'_> {
         Dump {
             memory: self,
             start: range.start_bound().cloned(),
@@ -362,7 +377,7 @@ impl<T> MemoryMap<'_, T> {
     #[doc(alias = "get_memory")]
     #[inline]
     pub fn memory(&self) -> &MemoryRef {
-        self.memory
+        unsafe { MemoryRef::from_ptr(self.map_info.memory) }
     }
 
     #[inline]
@@ -432,7 +447,7 @@ impl<T> Drop for MemoryMap<'_, T> {
     #[inline]
     fn drop(&mut self) {
         unsafe {
-            ffi::gst_memory_unmap(self.memory.as_mut_ptr(), &mut self.map_info);
+            ffi::gst_memory_unmap(self.map_info.memory, &mut self.map_info);
         }
     }
 }
@@ -458,15 +473,15 @@ impl<T> MappedMemory<T> {
     #[doc(alias = "get_memory")]
     #[inline]
     pub fn memory(&self) -> &MemoryRef {
-        self.memory.as_ref()
+        unsafe { MemoryRef::from_ptr(self.map_info.memory) }
     }
 
     #[inline]
     pub fn into_memory(self) -> Memory {
         let mut s = mem::ManuallyDrop::new(self);
-        let memory = unsafe { ptr::read(&s.memory) };
+        let memory = unsafe { from_glib_full(s.map_info.memory) };
         unsafe {
-            ffi::gst_memory_unmap(memory.as_mut_ptr(), &mut s.map_info);
+            ffi::gst_memory_unmap(s.map_info.memory, &mut s.map_info);
         }
 
         memory
@@ -517,7 +532,8 @@ impl<T> Drop for MappedMemory<T> {
     #[inline]
     fn drop(&mut self) {
         unsafe {
-            ffi::gst_memory_unmap(self.memory.as_mut_ptr(), &mut self.map_info);
+            let _memory = Memory::from_glib_full(self.map_info.memory);
+            ffi::gst_memory_unmap(self.map_info.memory, &mut self.map_info);
         }
     }
 }
@@ -949,6 +965,88 @@ macro_rules! memory_object_wrapper {
     };
 }
 
+#[cfg(feature = "v1_26")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_26")))]
+#[doc(alias = "GstMemory")]
+pub struct MemoryRefTrace(ffi::GstMemory);
+#[cfg(feature = "v1_26")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_26")))]
+impl MemoryRefTrace {
+    pub unsafe fn from_ptr<'a>(ptr: *mut ffi::GstMemory) -> &'a MemoryRefTrace {
+        assert!(!ptr.is_null());
+
+        &*(ptr as *const Self)
+    }
+
+    pub fn as_ptr(&self) -> *const ffi::GstMemory {
+        self as *const Self as *const ffi::GstMemory
+    }
+
+    #[doc(alias = "get_allocator")]
+    #[inline]
+    pub fn allocator(&self) -> Option<&Allocator> {
+        unsafe {
+            if self.0.allocator.is_null() {
+                None
+            } else {
+                Some(Allocator::from_glib_ptr_borrow(&self.0.allocator))
+            }
+        }
+    }
+
+    #[doc(alias = "get_parent")]
+    #[inline]
+    pub fn parent(&self) -> Option<&MemoryRef> {
+        unsafe {
+            if self.0.parent.is_null() {
+                None
+            } else {
+                Some(MemoryRef::from_ptr(self.0.parent))
+            }
+        }
+    }
+
+    #[doc(alias = "get_maxsize")]
+    #[inline]
+    pub fn maxsize(&self) -> usize {
+        self.0.maxsize
+    }
+
+    #[doc(alias = "get_align")]
+    #[inline]
+    pub fn align(&self) -> usize {
+        self.0.align
+    }
+
+    #[doc(alias = "get_offset")]
+    #[inline]
+    pub fn offset(&self) -> usize {
+        self.0.offset
+    }
+
+    #[doc(alias = "get_size")]
+    #[inline]
+    pub fn size(&self) -> usize {
+        self.0.size
+    }
+
+    #[doc(alias = "get_flags")]
+    #[inline]
+    pub fn flags(&self) -> crate::MemoryFlags {
+        unsafe { from_glib(self.0.mini_object.flags) }
+    }
+
+    #[doc(alias = "gst_memory_is_type")]
+    pub fn is_type(&self, mem_type: &str) -> bool {
+        unsafe {
+            from_glib(ffi::gst_memory_is_type(
+                self as *const Self as *mut ffi::GstMemory,
+                mem_type.to_glib_none().0,
+            ))
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     #[test]
