@@ -5,14 +5,14 @@ macro_rules! bitflags_serialize_impl {
     // this implementation serializes only flags using only one bit,
     // ignoring all other flags
     ($type:ty, single_bit_flags$(, $feature:expr)?) => {
-        $(#[cfg(any(feature = $feature, feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = $feature)))])?
+        $(#[cfg(any(feature = $feature, docsrs))]
+        #[cfg_attr(docsrs, doc(cfg(feature = $feature)))])?
         impl serde::Serialize for $type {
             fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
             where
                 S: serde::Serializer,
             {
-                let class = FlagsClass::new(Self::static_type()).unwrap();
+                let class = FlagsClass::with_type(Self::static_type()).unwrap();
                 let this = self.to_value();
 
                 let mut handled = Self::empty().to_value();
@@ -36,20 +36,20 @@ macro_rules! bitflags_serialize_impl {
 
     // considers the flags using the most bits first
     ($type:ty, by_ones_decreasing$(, $feature:expr)?) => {
-        $(#[cfg(any(feature = $feature, feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = $feature)))])?
+        $(#[cfg(any(feature = $feature, docsrs))]
+        #[cfg_attr(docsrs, doc(cfg(feature = $feature)))])?
         impl serde::Serialize for $type {
             fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
             where
                 S: serde::Serializer,
             {
-                use once_cell::sync::Lazy;
-
                 let mut handled = Self::empty();
                 let mut res = String::new();
 
-                static SORTED_VALUES: Lazy<Vec<(u32, String)>> = Lazy::new(|| {
-                    let class = FlagsClass::new(<$type>::static_type()).unwrap();
+                static SORTED_VALUES: std::sync::OnceLock<Vec<(u32, String)>> = std::sync::OnceLock::new();
+
+                let sorted_values = SORTED_VALUES.get_or_init(|| {
+                    let class = FlagsClass::with_type(<$type>::static_type()).unwrap();
                     let mut sorted_values: Vec<(u32, String)> =
                         class.values().iter()
                             .map(|f| (f.value(), f.nick().to_owned()))
@@ -62,7 +62,7 @@ macro_rules! bitflags_serialize_impl {
                     sorted_values
                 });
 
-                for (bits, nick) in SORTED_VALUES.iter() {
+                for (bits, nick) in sorted_values.iter() {
                     // not all values defined in the class are always also defined
                     // in the bitflags struct, see RTPBufferFlags for example
                     if let Some(value) = Self::from_bits(*bits) {
@@ -85,8 +85,8 @@ macro_rules! bitflags_serialize_impl {
 #[macro_export]
 macro_rules! bitflags_deserialize_impl {
     ($type:ty$(, $feature:expr)?) => {
-        $(#[cfg(any(feature = $feature, feature = "dox"))]
-        #[cfg_attr(feature = "dox", doc(cfg(feature = $feature)))])?
+        $(#[cfg(any(feature = $feature, docsrs))]
+        #[cfg_attr(docsrs, doc(cfg(feature = $feature)))])?
         impl<'de> serde::Deserialize<'de> for $type {
             fn deserialize<D: serde::de::Deserializer<'de>>(
                 deserializer: D,
@@ -114,7 +114,7 @@ macro_rules! bitflags_deserialize_impl {
 
                         let mut gvalue = unsafe { glib::Value::from_type_unchecked(Self::Value::static_type()) };
                         let tokens = value.split('+');
-                        let class = FlagsClass::new(Self::Value::static_type()).unwrap();
+                        let class = FlagsClass::with_type(Self::Value::static_type()).unwrap();
 
                         for token in tokens {
                             gvalue = class.set_by_nick(gvalue, token).map_err(|_| {
